@@ -9,7 +9,7 @@
  * in the Software without restriction, including without limitation the rights
  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following condition:
+ * furnished to do so, subject to the following conditions:
  * 
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
@@ -40,17 +40,10 @@ buffer* buf_new(unsigned int size) {
 /* free the buffer's data and the buffer itself */
 void buf_free(buffer* buf) {
 
-	buf_clear(buf);
+	m_free(buf->data)
 	m_free(buf);
 }
 
-/* free a buffer's data */
-void buf_clear(buffer* buf) {
-
-	m_free(buf->data);
-
-}
-
 /* overwrite the contents of the buffer to clear it */
 void buf_burn(buffer* buf) {
 	
@@ -84,16 +77,15 @@ void buf_resize(buffer *buf, unsigned int newsize) {
 }
 
 /* create a copy of buf, allocating required memory etc */
+/* the new buffer is sized the same as the length of the source buffer */
 /* lenonly is a boolean flag specifying whether to set the size of the new
  * buffer to be just the len of the source buffer (1), or the size of the
  * source buffer (0) */
-buffer* buf_newcopy(buffer* buf, int lenonly) {
+buffer* buf_newcopy(buffer* buf) {
 	
 	buffer* ret;
-	unsigned int newsize;
 
-	newsize = lenonly ? buf->len : buf->size;
-	ret = buf_new(newsize);
+	ret = buf_new(buf->len);
 	ret->len = buf->len;
 	memcpy(ret->data, buf->data, buf->len);
 	return ret;
@@ -107,7 +99,7 @@ void buf_setlen(buffer* buf, unsigned int len) {
 }
 
 void buf_incrlen(buffer* buf, unsigned int incr) {
-	if (buf->pos + incr > buf->size) {
+	if (buf->len + incr > buf->size) {
 		dropbear_exit("bad buf_incrlen");
 	}
 	buf->len += incr;
@@ -220,8 +212,7 @@ void buf_putint(buffer* buf, int unsigned val) {
 void buf_putstring(buffer* buf, const unsigned char* str, unsigned int len) {
 	
 	buf_putint(buf, len);
-	memcpy(buf_getwriteptr(buf, len), str, len);
-	buf_incrwritepos(buf, len);
+	buf_putbytes(buf, str, len);
 
 }
 
@@ -250,6 +241,8 @@ void buf_putmpint(buffer* buf, mp_int * mp) {
 	if (USED(mp) == 1 && DIGIT(mp, 0) == 0) {
 		len = 0;
 	} else {
+		/* SSH spec requires padding for mpints with the MSB set, this code
+		 * implements it */
 		len = mp_count_bits(mp);
 		/* if the top bit of MSB is set, we need to pad */
 		pad = (len%8 == 0) ? 1 : 0;
@@ -275,9 +268,9 @@ void buf_putmpint(buffer* buf, mp_int * mp) {
 	TRACE(("leave buf_putmpint"));
 }
 
-/* retrieve an mp_int from the buffer. This will only handle +ve or 0 values, 
- * will fail for -ve since they shouldn't be required here.
- * Returns 0 on success, -1 on failure */
+/* Retrieve an mp_int from the buffer.
+ * Will fail for -ve since they shouldn't be required here.
+ * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 int buf_getmpint(buffer* buf, mp_int* mp) {
 
 	unsigned int len;
@@ -285,18 +278,18 @@ int buf_getmpint(buffer* buf, mp_int* mp) {
 	
 	if (len == 0) {
 		mp_zero(mp);
-		return 0;
+		return DROPBEAR_SUCCESS;
 	}
 
 	/* check for negative */
 	if (*buf_getptr(buf, 1) & (1 << (CHAR_BIT-1))) {
-		return -1;
+		return DROPBEAR_FAILURE;
 	}
 
 	if (mp_read_unsigned_bin(mp, buf_getptr(buf, len), len) != MP_OKAY) {
-		return -1;
+		return DROPBEAR_FAILURE;
 	}
 
 	buf_incrpos(buf, len);
-	return 0;
+	return DROPBEAR_SUCCESS;
 }
