@@ -28,6 +28,7 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include "loginrec.h"
+#include "channel.h"
 
 struct ChanSess {
 
@@ -38,7 +39,6 @@ struct ChanSess {
 	int master; /* the master terminal fd*/
 	int slave;
 	unsigned char * tty;
-
 	unsigned char * term;
 	unsigned int termw, termh, termc, termr; /* width, height, col, rows */
 
@@ -48,6 +48,20 @@ struct ChanSess {
 	int exitsignal;
 	unsigned char exitcore;
 	
+#ifndef DISABLE_X11FWD
+	int x11fd; /* set to -1 to indicate forwarding not established */
+	int x11port;
+	char * x11authprot;
+	char * x11authcookie;
+	unsigned int x11screennum;
+	unsigned char x11singleconn;
+#endif
+
+#ifndef DISABLE_AGENTFWD
+	int agentfd;
+	char * agentfile;
+	char * agentdir;
+#endif
 };
 
 struct ChildPid {
@@ -64,6 +78,7 @@ void send_msg_chansess_exitstatus(struct Channel * channel,
 		struct ChanSess * chansess);
 void send_msg_chansess_exitsignal(struct Channel * channel,
 		struct ChanSess * chansess);
+void addnewvar(const char* param, const char* var);
 
 
 
