@@ -1,3 +1,27 @@
+/*
+ * Dropbear - a SSH2 server
+ * 
+ * Copyright (c) 2002,2003 Matt Johnston
+ * All rights reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE. */
+
 #include <stdlib.h>
 #include <string.h>
 
@@ -23,7 +47,7 @@ sign_key * new_sign_key() {
 
 }
 
-/* returns 0 on success, -1 on fail */
+/* returns DROPBEAR_SUCCESS on success, DROPBEAR_FAILURE on fail */
 int buf_get_pub_key(buffer *buf, sign_key *key, int type) {
 
 	unsigned char* ident;
@@ -54,8 +78,7 @@ int buf_get_pub_key(buffer *buf, sign_key *key, int type) {
 
 	m_free(ident);
 
-	dropbear_msg("No valid key type found");
-	return -1;
+	return DROPBEAR_FAILURE;
 	
 }
 
@@ -96,7 +119,6 @@ int buf_get_priv_key(buffer *buf, sign_key *key, int type) {
 
 	m_free(ident);
 	
-	dropbear_msg("No valid key type found");
 	TRACE(("leave buf_get_priv_key"));
 	return -1;
 	
@@ -200,8 +222,9 @@ void buf_put_sign(buffer* buf, sign_key *key, int type,
 }
 
 #ifdef DROPBEAR_SIGNKEY_VERIFY
-/* Return 1 if good signature, 0 otherwise. If 0 is returned, the position of
- * buf is undefined. If 1 is returned, buf will be positioned after the
+/* Return DROPBEAR_SUCCESS or DROPBEAR_FAILURE.
+ * If FAILURE is returned, the position of
+ * buf is undefined. If SUCCESS is returned, buf will be positioned after the
  * signature blob */
 int buf_verify(buffer * buf, sign_key *key, const unsigned char *data,
 		unsigned int len) {
@@ -212,6 +235,7 @@ int buf_verify(buffer * buf, sign_key *key, const unsigned char *data,
 
 	bloblen = buf_getint(buf);
 	ident = buf_getstring(buf, &identlen);
+
 #ifdef DROPBEAR_DSS
 	if (bloblen == DSS_SIGNATURE_SIZE &&
 			memcmp(ident, SSH_SIGNKEY_DSS, identlen) == 0) {
@@ -219,14 +243,16 @@ int buf_verify(buffer * buf, sign_key *key, const unsigned char *data,
 		return buf_dss_verify(buf, key->dsskey, data, len);
 	}
 #endif
+
 #ifdef DROPBEAR_RSA
 	if (memcmp(ident, SSH_SIGNKEY_RSA, identlen) == 0) {
 		m_free(ident);
 		return buf_rsa_verify(buf, key->rsakey, data, len);
 	}
 #endif
+
 	m_free(ident);
 	dropbear_exit("non-matching signing type");
-	return 0;
+	return DROPBEAR_FAILURE;
 }
 #endif /* DROPBEAR_SIGNKEY_VERIFY */
