@@ -9,7 +9,7 @@
  * in the Software without restriction, including without limitation the rights
  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following condition:
+ * furnished to do so, subject to the following conditions:
  * 
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
@@ -20,7 +20,33 @@
  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE. */
+ * SOFTWARE.
+ *
+ * strlcat() is copyright as follows:
+ * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The name of the author may not be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
+ * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
+ * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+ * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+ * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 
 #include <stdio.h>
 #include <unistd.h>
@@ -48,7 +74,7 @@ void startsyslog() {
 
 	int fd;
 
-	openlog(PROGNAME, LOG_PID, LOG_DAEMON);
+	openlog(PROGNAME, LOG_PID, LOG_AUTHPRIV);
 
 #ifndef DEBUG_TRACE
 	/* redirect stdin/stdout/stderr to /dev/null */
@@ -89,19 +115,26 @@ void dropbear_exit(const char* format, ...) {
 /* failure exit - format must be <= 100 chars */
 static void _dropbear_exit(int exitcode, const char* format, va_list param) {
 
-#define EXIT_MESSAGE "exited: "
-#define EXIT_MESSAGE_LEN 8
+	char fmtbuf[200];
 
-	char fmtbuf[MAX_FMT+EXIT_MESSAGE_LEN+1];
+	if (!sessinitdone) {
+		/* before session init */
+		snprintf(fmtbuf, sizeof(fmtbuf), "exited: %s", format);
+	} else if (ses.authstate.authdone) {
+		/* user has authenticated */
+		snprintf(fmtbuf, sizeof(fmtbuf),
+				"exited after userauth (%s) from %s: %s", 
+				ses.authstate.printableuser, ses.addrstring, format);
+	} else {
+		/* before userauth */
+		snprintf(fmtbuf, sizeof(fmtbuf), "exited before userauth: %s",
+				format);
+	}
 
-#ifdef DOCLEANUP
+	_dropbear_log(LOG_INFO, fmtbuf, param);
+
+	/* must be after using username etc */
 	session_cleanup();
-#endif
-
-	strcpy(fmtbuf, EXIT_MESSAGE);
-	strncat(fmtbuf, format, MAX_FMT);
-
-	_dropbear_log(LOG_DAEMON | LOG_INFO, fmtbuf, param);
 
 	exit(exitcode);
 
@@ -120,14 +153,15 @@ void dropbear_log(int priority, const char* format, ...) {
 /* priority is priority as with syslog() */
 static void _dropbear_log(int priority, const char* format, va_list param) {
 
-	if (usingsyslog) {
-		vsyslog(priority, format, param);
-	} else {
-		fprintf(stderr, "dropbear ");
-		vfprintf(stderr, format, param);
-		fprintf(stderr, "\n");
-	}
+	char printbuf[1024];
 
+	vsnprintf(printbuf, sizeof(printbuf), format, param);
+
+	if (usingsyslog) {
+		syslog(priority, printbuf);
+	} else {
+		fprintf(stderr, "dropbear %s\n", printbuf);
+	}
 }
 
 #ifdef DEBUG_TRACE
@@ -143,6 +177,43 @@ void dropbear_trace(const char* format, ...) {
 }
 #endif /* DEBUG_TRACE */
 
+/* Return a string representation of the socket address passed. The return
+ * value is allocated with strdup() */
+unsigned char * getaddrstring(struct sockaddr * addr) {
+
+	switch (addr->sa_family) {
+		case PF_INET: 
+			return strdup(inet_ntoa(((struct sockaddr_in *)addr)->sin_addr));
+	}
+
+	/* Need to extend for IP6 */
+	return NULL;
+
+}
+
+/* Get the hostname corresponding to the address addr. On failure, the IP
+ * address is returned. The return value is allocated with strdup() */
+char* getaddrhostname(struct sockaddr * addr) {
+
+	struct hostent *host = NULL;
+	char * retstring;
+
+#ifdef DO_HOST_LOOKUP
+	host = gethostbyaddr((char*)&((struct sockaddr_in*)addr)->sin_addr,
+			sizeof(struct in_addr), AF_INET);
+#endif
+	
+	if (host == NULL) {
+		/* return the address */
+		retstring = inet_ntoa(((struct sockaddr_in *)addr)->sin_addr);
+	} else {
+		/* return the hostname */
+		retstring = host->h_name;
+	}
+
+	return strdup(retstring);
+}
+
 #ifndef HAVE_STRLCPY
 /* Implemented by matt as specified in freebsd 4.7 manpage.
  * We don't require great speed, is simply for use with sshpty code */
@@ -169,6 +240,49 @@ size_t strlcpy(char *dst, const char *src, size_t size) {
 }
 #endif /* HAVE_STRLCPY */
 
+#ifndef HAVE_STRNCAT
+/* taken from openbsd-compat for OpenSSH 3.6.1p1 */
+/* "$OpenBSD: strlcat.c,v 1.8 2001/05/13 15:40:15 deraadt Exp $"
+ *
+ * Appends src to string dst of size siz (unlike strncat, siz is the
+ * full size of dst, not space left).  At most siz-1 characters
+ * will be copied.  Always NUL terminates (unless siz <= strlen(dst)).
+ * Returns strlen(src) + MIN(siz, strlen(initial dst)).
+ * If retval >= siz, truncation occurred.
+ */
+size_t
+strlcat(dst, src, siz)
+	char *dst;
+	const char *src;
+	size_t siz;
+{
+	register char *d = dst;
+	register const char *s = src;
+	register size_t n = siz;
+	size_t dlen;
+
+	/* Find the end of dst and adjust bytes left but don't go past end */
+	while (n-- != 0 && *d != '\0')
+		d++;
+	dlen = d - dst;
+	n = siz - dlen;
+
+	if (n == 0)
+		return(dlen + strlen(s));
+	while (*s != '\0') {
+		if (n != 1) {
+			*d++ = *s;
+			n--;
+		}
+		s++;
+	}
+	*d = '\0';
+
+	return(dlen + (s - src));	/* count does not include NUL */
+}
+#endif /* HAVE_STRLCAT */
+
+#ifdef DEBUG_TRACE
 void printhex(unsigned char* buf, int len) {
 
 	int i;
@@ -184,6 +298,35 @@ void printhex(unsigned char* buf, int len) {
 	}
 	fprintf(stderr, "\n");
 }
+#endif
+
+/* Strip all control characters from text (a null-terminated string), except
+ * for '\n', '\r' and '\t'.
+ * The result returned is a newly allocated string, this must be free()d after
+ * use */
+char * stripcontrol(const char * text) {
+
+	char * ret;
+	int len, pos;
+	int i;
+	
+	len = strlen(text);
+	ret = m_malloc(len+1);
+
+	pos = 0;
+	for (i = 0; i < len; i++) {
+		if ((text[i] <= '~' && text[i] >= ' ') /* normal printable range */
+				|| text[i] == '\n' || text[i] == '\r' || text[i] == '\t') {
+			ret[pos] = text[i];
+			pos++;
+		}
+	}
+	ret[pos] = 0x0;
+	return ret;
+}
+			
+
+
 
 /* returns the current position on success, or -1 on failure */
 int readln(int fd, char* buf, int count) {
